#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <fstream>
using namespace std;


// ======================================================================
typedef unsigned int nb_chars;

/* Libre à vous de créer ici VOTRE modèle,
 * mais CREEZ EN UN !
 */
struct Sapin {
  vector<nb_chars> sizes;    // taille des trapezes
  nb_chars trunk;            // hauteur du tronc
  char body;                  // char pour dessiner le corps du sapin
  char wood;                  // char pour le tronc (made of wood, isn't it?)
  bool decorate;              // sapin de Noël ?
};

constexpr char boule('O'); // est-ce que les boules font partie du sapin ?? (plutôt non)
// Volontairement fait un choix externe ici pour montrer autre chose.
// On pourrait aussi créer une struct Decoration, par exemple...

// ======================================================================
string demander_nom()
{
  string retour;

  cout <<  "Entrez un nom de fichier : ";
  cin >> retour;
  
  return retour;
}

// ======================================================================
bool ouvrir(ofstream& fichier)
{
  const string nom(demander_nom());
  fichier.open(nom);

  // NE PAS OUBLIER
  if (fichier.fail()) {
    cerr << "Impossible d'ouvrir le fichier \"" << nom
         << "\" en écriture." << endl;
    return false;
  }
  
  return true;
}
  
// ======================================================================
// répète `size` fois (au moins une) le caractère `c` après une marge de `offset`
// (et passe à la ligne)
void repeter(ofstream& out, nb_chars offset, nb_chars size, char c = '.')
{
  out << setfill(' ') << setw(offset+1) << c;
  if (size > 1) {
    out << setfill(c) << setw(size-1) << c;
  }
  out << endl;
}

// ======================================================================
void dessiner_trapeze_simple(ofstream& out, nb_chars offset, nb_chars size,
                             char corps = 'X')
{
  for (nb_chars i(1); i < size; ++i)
    repeter(out, offset+size-i, 2*i+1, corps);
}

// ======================================================================
void dessiner_trapeze(ofstream& out, nb_chars offset, nb_chars size,
                      char decoration = '+', char corps = '+')
{
  constexpr nb_chars mini_decoration(5);
  
  for (nb_chars i(1); i < size; ++i) {
    // decoration ou pas ?
    if ((decoration != corps) and (size >= mini_decoration)
        and (i > (mini_decoration - 2) and (i%2 == 0))) {
      out << setfill(' ') << setw(offset+size-i+1) << corps;
      const nb_chars step(i/2+i%5);
      nb_chars width(2*i);
      for (; width >= step; width -= step) {
        out << setfill(corps) << setw(step) << decoration;
      }
      if (width >0) {
        out << setfill(corps) << setw(width) << corps;
      }
      out << endl;
    } else {
      // sans décoration
      repeter(out, offset+size-i, 2*i+1, corps);
    }
  }
}

// ======================================================================
void dessiner_fleche(ofstream& out, nb_chars decalage = 0)
{
  repeter(out, decalage, 1, '^');
  repeter(out, decalage, 1, '|');
}

// ======================================================================

// aussi possible : 
// bool afficher_sapin(ofstream& sortie, Sapin s) : pour indiquer l'échec

void afficher_sapin(ofstream& out, Sapin const& sapin, nb_chars marge = 0)
{
  const nb_chars decalage(marge + sapin.sizes.back());

  // flêche
  if (sapin.decorate) dessiner_fleche(out, decalage);

  // corps
  repeter(out, decalage, 1, sapin.body); // la pointe
  for(auto taille : sapin.sizes) {
    dessiner_trapeze(out, decalage - taille, taille,
                     sapin.decorate ? boule : sapin.body,
                     sapin.body);
  }

  // tronc
  const nb_chars trunk_size(2 + sapin.sizes.back() / 3);
  for (nb_chars i(1); i <= sapin.trunk; ++i) {
    repeter(out, decalage - trunk_size / 2, trunk_size, sapin.wood);
  }
}

// ======================================================================
int main()
{
  ofstream sortie;
  if (ouvrir(sortie)) {
    afficher_sapin(sortie,
                   { { 3, 5, 9 }, 3, '*', '#', true }, // un joli sapin de Noël
                   4 // marge : là où « planter » le sapin
                   );
    sortie.close();
  }
  
  return 0;
}
